# ESA (C) 2000-2022
# 
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

# RunCifbuild.py


import glob
import os
from pysas.logger import TaskLogger as TL
logger = TL('CifBuild')
from pysas.pyutils import pyutils
import sys
from pysas.wrapper import Wrapper as W



def set_ccf(ccf):
    """
    Sets the environment variable SAS_CCF to the given ccf.

    Args:
        the proper CCF.
    """

    os.environ['SAS_CCF'] = ccf
    logger.log('info', 'SAS_CCF: {}'.format(os.environ['SAS_CCF']))


def check_HEASOFT():
    """
    Check if HEASOFT has been exported in order to use SAS.
    """

    logger.log('info', 'Checking for HEASOFT...')
    
    if not 'HEADAS' in os.environ:
        logger.log('error', 'HEADAS environment variable does not exist\n')
        logger.log('error', 'Please, set the HEADAS enviroment variable and initialize the software.')
#        ErrorCode.error_code(2)
    else:
        logger.log('info', 'HEADAS = {} ... DONE'.format(os.environ['HEADAS']))


def set_odfingest(odf):
    """
    Sets the environment variable SAS_ODF.

    Args:
        odf: the proper ODF.
    """

    os.environ['SAS_ODF'] = odf
    logger.log('info', 'SAS_ODF: {}'.format(os.environ['SAS_ODF']))


def check_SASODF():
    """
    Checks the SAS_ODF environment variable.
    """

    logger.log('info', 'Checking existance of SAS_ODF environment variable...')

    if not 'SAS_ODF' in os.environ:
        logger.log('warning', 'SAS_ODF environment variable does not exist ... ')
        logger.log('warning', 'Please, set the SAS_ODF enviroment variable. By default, this directory will be chosen or the SAS file will be chosen.')
        SAS_search = glob.glob('*.SAS')
        if len(SAS_search) > 0:
            os.environ['SAS_ODF'] = os.path.abspath(SAS_search[0])
        elif len(glob.glob('*FIT')) > 0:
            os.environ['SAS_ODF'] = os.getcwd()
        else:
            logger.log('error', 'Could not set the SAS_ODF variable.')
            raise FileNotFoundError('Could not set the SAS_ODF variable')
    else:
        if os.path.isfile(os.environ['SAS_ODF']):
            logger.log('debug', 'SAS_ODF = {} ... DONE'.format(os.environ['HEADAS']))
        elif os.path.isdir(os.environ['SAS_ODF']):
            logger.log('debug', 'SAS_ODF = {} ... DONE'.format(os.environ['HEADAS']))
        else:
            logger.log('warning', 'The current path in SAS_ODF does not containing a SUM.SAS file or a valid folder.')
            os.environ['SAS_ODF'] = os.getcwd()
            logger.log('error', '{} does not exist.'.format(os.environ['SAS_ODF']))
            raise FileNotFoundError('SAS_ODF file does not exist.')
#            ErrorCode.error_code(1)


def test_odfingest():
    """
    Check if any SUM_SAS file exists in the given analysis directory.
    """

    logger.log('info', 'Checking existance of SUM.SAS file ...')
    cwd = os.getcwd()
    os.chdir(cwd)
    SAS_list = glob.glob('*SUM.SAS')

    if len(SAS_list) != 0:
        odfingest_file = cwd + '/' + SAS_list[0]
        logger.log('debug', 'File found')
        return (1, odfingest_file)
    else:
        logger.log('debug', 'SUM.SAS file DOES NOT EXIST ..... Running')
        return (0, 'No file')


def test_CCF():
    """
    Checks if a ccf file exists.
    """

    logger.log('debug', 'Checking existance of CCF file ...')
    cwd = os.getcwd()
    os.chdir(cwd)
    ccffile = cwd + '/ccf.cif'
    
    if os.path.isfile(ccffile):
        logger.log('info', 'File found.')
        return (1, ccffile)
    else:
        logger.log('info', 'CCF file DOES NOT EXIST ... Running ...')
        return(0, 'No file.')


def run_odfingest():
    """
    Runs odfingest.
    """

    odfingest_task = W('odfingest', [])
    odfingest_task.run()

#    if PythonUtils.call_SAS('odfingest'):
#        ErrorCode.error_code(2)


def run_cifbuild():
    """
    Runs, if needed, cifbuild, after checking whether or not there's a prior
    ccf file.
    """

    check_SASODF()
    check_HEASOFT()

    cwd = os.getcwd()

    logger.log('info', 'Running cifbuild...')
    # print_sas_setup

    test_ccf = test_CCF()

    ccffile = test_ccf[1]

    if test_ccf[0]:
        logger.log('info', 'ccf file has already been produced...')
        set_ccf(ccffile)
        logger.log('debug', 'ccf file: {}'.format(ccffile))
        logger.log('debug', 'cifbuild OK:')
    else:
        inargs = ['withccfpath=yes', 'ccfpath={}'.format(os.environ['SAS_CCFPATH']), 'fullpath=yes']
        
        cifbuild_task = W('cifbuild', inargs)
        cifbuild_task.run()

        # Test after cifbuild:
        test_ccf = test_CCF()
        ccffile = test_ccf[1]

        if test_ccf[0]:
            set_ccf(ccffile)
            logger.log('debug', 'ccf file: {}'.format(ccffile))
            logger.log('debug', 'cifbuild OK:')
        else:
            logger.log('error' ,'file {} does not exist. Exiting...'.format(ccffile))
            raise FileNotFoundError('Could not find a proper CCF file.')
#            ErrorCode.error_code(2)

    return 0


def run_and_check_odfingest():
    '''
    Runs and checks variables for odfingest.
    '''

    logger.log('info', '\n Running odfingest .....')

    odfingest_test = test_odfingest()
    odfingest_file = odfingest_test[1]

    if odfingest_test[0] == 0:
        run_odfingest()
        odfingest_test = test_odfingest()
        odfingest_file = odfingest_test[1]

    if odfingest_test[0]:
        logger.log('debug', 'Summary file: {}'.format(odfingest_file))
        set_odfingest(odfingest_file)
        logger.log('info', 'odfingest OK')

    else:
        logger.log('error', 'odfingest file does not exist .... Exiting.')
        raise FileNotFoundError('Could not find odfingest product file.')
#        ErrorCode.error_code(2)

    logger.log('info', 'Working out cifbuild and odfinget. Finished')
#    ErrorCode.error_code(0)
