# ESA (C) 2000-2022
# 
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

# pyOAL.py

from .version import VERSION, SAS_RELEASE, SAS_AKA

__version__ = f'pyOAL (pyutils-{VERSION}) [{SAS_RELEASE}-{SAS_AKA}]'


import xml.etree.ElementTree as ET
import pysas.pyutils.pyutils as pyutils
import os
import tarfile
from pysas.wrapper import Wrapper as W
from astroquery.esa.xmm_newton import XMMNewton
import glob
from pysas.RunCifbuild import RunCifbuild


# xmlf = 0
#
# for file in os.listdir(os.getcwd()): # assuming the event files are in the cwd:
#     if '.xml' in file or '.XML' in file:
#         filename = file
#         xmlf = xmlf + 1
#
# if xmlf == 0:
#     raise OSError("No XML file found in current directory")
# elif xmlf > 1:
#     raise OSError("More than one XML file found in current directory")
# else:
#     print("Using event file: {}\n".format(filename))


def brancher(branch):
    '''
    Shows all the available content in a branch. The code reaches until 6,
    which should be the maximum for this XML.
    
    Args:
        branch: the XML branch (the selected_(...) attributes).
    '''

    if len(branch) != 0:
        for child in branch:
            print(child.tag, child.attrib)
            if len(child) != 0:
                for child2 in child:
                    print('  ', child2.tag, child2.attrib)
                    if len(child2) != 0:
                        for child3 in child2:
                            print('    ', child3.tag, child3.attrib)
                            if len(child3) != 0:
                                for child4 in child3:
                                    print('      ', child4.tag, child4.attrib)
                                    if len(child4) != 0:
                                        for child5 in child4:
                                            print('        ', child5.tag, child5.attrib)
                                            if len(child5) != 0:
                                                for child6 in child5:
                                                    print('          ', child6.tag, child6.attrib)



def check_if_valid_OBSID(obsid):
    '''
    Check if the given OBSID is a valid OBSID candidate.

    Args:
        obsid: the OBSID.

    Output:
        boolean.
    '''

    if len(obsid) == 10:
        return True
    else:
        return False
    # TODO add more detail?


def from_obsid(input_obs):
    '''
    Preepares the filename needed for the Observation class if an OBSID is given.

    Args:
        input_obs: the OBSID that has been downloaded.
    '''

    obs_file = input_obs + '.tar.gz'

    if os.path.isfile(obs_file):
        targz_file = tarfile.open(obs_file, 'r:gz')
        targz_file.extractall()
        targz_file.close()

        tar_file = glob.glob('*.TAR')

        if len(tar_file) != 0:
            tar_file = tar_file[0]
            tar = tarfile.open(tar_file, 'r:')
            tar.extractall()
            tar.close()

            # cifbuild and odfingest
            RunCifbuild.run_cifbuild()
            RunCifbuild.run_and_check_odfingest()

            # odfParamCreator
            input_obs_xml = input_obs + '.xml'
            run_param = W('odfParamCreator', ['outputFileName={}'.format(input_obs_xml)])
            run_param.run()
            filename = input_obs_xml
        else:
            raise FileNotFoundError('Could not locate .TAR file.')
    else:
        raise FileNotFoundError('Could not locate {}.'.format(obs_file))

    return filename


def from_folder(path_to_folder):
    '''
    Prepares the observation if a path to a folder is given.

    Args:
        path_to_folder: the path to the folder. 
    '''
    try:
        sas_file = glob.glob(path_to_folder + '*.SUM.SAS')[0]
    except IndexError:
        raise FileNotFoundError('Could not locate a summary file in folder {}.'.format(path_to_folder))

    inargs = [f'sas_ccf={path_to_folder}/ccf.cif', f'sas_odf={sas_file}/2466_0723801701_SCX00000SUM.SAS', f'workdir={work_dir}']    

    t = W('starsas', inargs)
    t.run()

    filename = 'parsedparams.xml'
    t = W('odfParamCreator', ['outputFileName={}'.format(filename)])
    t.run()

    return filename


def from_SUM_SAS(SUM_SAS):
    '''
    Prepares the observation if a path to the summary file was given.

    Args:
        SUM_SAS: the path to the summary file.

    Output:
        filename: the filename of the XML file ethat will be used to build the OBservation class.
    '''

    run_param = W('odfParamCreator', ['outputFileName={}'.format(input_obs_xml)])
    filename = os.path.basename(os.path.splitext(input_obs)[0])

    return filename


#############################################
################# Classes ###################
#############################################

class Observation:
    '''
    The class will contain the observation info for the XMMextractorGUI.
    Needs the path of the needed XML file or the OBSID.
    Attribute id: the basic id for an observation.
    Attribute SAS_version: the current SAS version.
    The attributes with parsed_ at the begining contain the XML branches as
    lists. The ones that don't have parsed_ contain simply the information as
    the XML dictionary. The id attribute refers to the basic information of the
    given observation in the .xml file.
    '''

    def __init__(self, input_obs = None, working_dir = None, epn = True, mos = True, rgs = True, om = True):
        '''
        Basic __init__ class for Observation.
        '''

        if working_dir is None:
            working_dir = os.getcwd()

        if input_obs == None:
            input_obs = os.getcwd()

        input_obs = str(input_obs)

        if input_obs.upper().endswith('XML'):
            if os.path.isfile(input_obs):
                filename = input_obs
            else:
                raise FileNotFoundError('Could not find the given XML file.')
        elif input_obs.upper().endswith('SAS'):
            if os.path.isfile(input_obs):
                filename = from_SUM_SAS(input_obs)
            else:
                raise FileNotFoundError('Could not  find the given SAS file.')
        elif os.path.isdir(input_obs):
                filename = from_folder(input_obs)
        else: # no filename but OBSID
            is_valid_obs = check_if_valid_OBSID(input_obs)
            if not is_valid_obs:
                raise TypeError('Invalid OBSID number.')
            try:
                XMMNewton.download_data(input_obs, level = 'ODF')
            except:
                raise Exception('The requested OBSID is private or does not exist.')
            filename = from_obsid(input_obs)            


        global tree
        tree = ET.parse(filename)
        self.tree = tree
        global root
        root = self.tree.getroot()

        config = root[0]
        SAS_version = config.attrib['SASVersion']
        obs = config[0]


        full_obs = dict()

        for i in obs:
            full_obs.update({i.attrib['id'] : i.attrib['default']})

        self.full_obs = full_obs
        self.ra = full_obs['ra']
        self.dec = full_obs['dec']
        self.EMOS1 = full_obs['EMOS1']
        if mos and self.EMOS1 == 'yes':
            self.EMOS1 = True
        else:
            self.EMOS1 = False
        self.EPN = full_obs['EPN']
        if epn and self.EPN == 'yes':
            self.EPN = True
        else:
            self.EPN = False
        self.EMOS2 = full_obs['EMOS2']
        if mos and self.EMOS2 == 'yes':
            self.EMOS2 = True
        else:
            self.EMOS2 = False
        self.OM = full_obs['OM']
        if om and self.OM == 'yes':
            self.OM = True
        else:
            self.OM = False
        self.analysisoption = full_obs['analysisoption']
        self.RGS1 = full_obs['RGS1']
        if rgs and self.RGS1 == 'yes':
            self.RGS1 = True
        else:
            self.RGS1 = False
        self.RGS2 = full_obs['RGS2']
        if rgs and self.RGS2 == 'yes':
            self.RGS2 = True
        else:
            self.RGS2 = False
        self.OM_sourcematch = full_obs['OM_sourcematch']
        self.obsid = full_obs['obsid']
        self.sourcename = full_obs['sourcename']
        self.epicsrc = full_obs['epicsrc']

        
        parsed_instruments = []
        
#        for i in range(1, len(config)):
#            print(config[i])
#            parsed_instruments.append(config[i])

        # lists are created
        instrument_list = []
        parsed_exposures = []
        exposure_list = []
        parsed_products = []
        product_list = []
        parsed_tasks = []
        task_list = []
        parsed_params = []
        param_list = []

        # lists are filled. The proper list contain easy to read information.
        # The parsed lists contain the branches of the XML.

        parsed_instruments = []
        for i in range(1, len(config)):
            instr_flag = getattr(self, config[i].attrib['value'])
            if instr_flag:
                parsed_instruments.append(config[i])
                instrument_list.append((config[i].tag, config[i].attrib))
                for child in config[i]:
                    parsed_exposures.append(child)
                    exposure_list.append(Exposure(config[i], child))
                    for child2 in child:
                        parsed_products.append(child2)
                        product_list.append(Product(config[i], child, child2))
                        for child3 in child2:
                            parsed_tasks.append(child3)
                            task_list.append(Task(config[i], child, child2, child3))
                            for child4 in child3:
                                parsed_params.append(child4)
                                param_list.append(Param(config[i], child, child2, child3, child4))
            else:
                pass                


        # Filling extra attributes with information
        self.SAS_version = SAS_version
        self.instruments = instrument_list
        self.parsed_instruments = parsed_instruments
        self.exposures = exposure_list
        self.parsed_exposures = parsed_exposures
        self.products = product_list
        self.parsed_products = parsed_products
        self.tasks = task_list
        self.parsed_tasks = parsed_tasks
        self.params = param_list
        self.parsed_params = parsed_params
        self.working_dir = working_dir


    def parent_child_map(self):
        '''
        Simply returns the map with all the parent/child relations.
        
        Output:
            parent_map: the map as a dictionary.
        '''

        parent_map = {c:p for p in tree.iter() for c in p}
        return parent_map


    def set_coord(self, ra = '', dec = ''):
        '''
        Change the value of the RA and DEC for an observation.
        This file does NOT rewrite a xml or the existing xml file.
        
        Args:
            ra: the desired change for ra
            dec: the desired change for dec.
        '''

        for child in tree.iterfind('CONFIG/OBSERVATION'):
            for i in child:
                if ra == '':
                    pass
                else:
                    if i.attrib['id'] == 'ra':
                        i.attrib['default'] = ra
                        self.ra = ra
                        self.full_obs['ra'] = ra
                if dec == '':
                    pass
                else:
                    if i.attrib['id'] == 'dec':
                        i.attrib['default'] = dec
                        self.dec = dec
                        self.full_obs['dec'] = dec


    def check_instruments(self, epn, mos, rgs, om):
        '''
        Since by default all instruments are available, checks in the 
        original XML whether or not this is true and modifies the id.

        Also considers the input from XMMextractorpy.

        Args:
            epn: boolean - whether or not to use EPN.
            mos: boolean - whether or not to use EMOS.
            rgs: boolean - whether or not to use RGS.
            om: boolean: whetheere or not to use OM.
        '''

        real_instruments = []
        for i in self.instruments:
            real_instruments.append(i[1]['value'])

        for i in self.ids:
            if not i[1]['id'] in real_instruments and i[1]['id'] in ['EPN', 'EMOS1', 'EMOS2', 'RGS1', 'RGS2', 'OM']:
                self.id[i][1]['id'] = 'no'


    def get_instrument(self, selection = ''):
        ''' 
        Returns either information regarding all instruments or a single
        instrument as from the child class.
        
        Args:
            selection  (optional): the name of the instrument.
        
        Output: 
            either the full list of instruments or the instrument that
        fulfilled the given condition.
        '''

        all_instruments = []
        if selection == '':
            for child in self.parsed_instruments:
                all_instruments.append(Instrument(child))
            return all_instruments
        elif selection != '':
            for child in self.parsed_instruments:
                if selection in child.attrib['value']:
                    return Instrument(child)


    def get_exposures_obs(self, expid = ''):
        '''
        Returns either a full list of the exposures available for the
        observation or a single Exposure class object if a selection was given.
        
        Args:
            (optional): expid of the exposition(s).
        
        Output:
            either the whole list of exposures, a list with all the 
            exposures that match the condition or if the list only has 
            one element, it will directly create an object of Exposure class.
        '''

        selection_list = []

        if expid == '':
            return self.exposures
        else:
            for exposure in self.exposures:
                if expid in exposure.expid:
                    selection_list.append(exposure)

        if len(selection_list) == 1:
            return selection_list[0]
        else:
            return selection_list


    def get_products_obs(self, selection = ''):
        '''
        Gets all products from the observation that match a selection. If
        no selection is given, all the products will be returned.
        

        Args:
            selection (optional): the name of the product.
        
        Output:
            either all the products or a list containing XML branches with
        the products.
        '''

        selection_list = []

        if selection == '':
            return self.products
        else:
            for child in self.products:
                if selection in child.full_product[1]['value']:
                    selection_list.append(child)

        return selection_list


    def get_tasks_obs(self, name = '', purpose = ''):
        '''
        Selects certain tasks from the observation.
        
        Args:
            name: the name of the task.
            purpose: the purpose of the task.
        
        Output: 
            a list with the XML branch with all the branches that made
        the cut. NOT a Task object.
        '''

        selection_list = []


        if name == '' and purpose == '':
            return self.tasks
        if name != '' and purpose == '':
            for child in self.tasks:
                if name in child.full_task[1]['name']:
                    selection_list.append(child)
        if name == '' and purpose != '':
            for child in self.tasks:
                if purpose in child.full_task[1]['purpose']:
                    selection_list.append(child)
        if name != '' and purpose != '':
            for child in self.tasks:
                if name in child.full_task[1]['name'] and purpose in child.full_task[1]['purpose']:
                    selection_list.append(child)

        return selection_list


    def get_params_obs(self, param_name = ''):
        '''
        Returns parameters with the given name. If no name is given, it'll simply
        return them all. Which I'm not sure myself if it makes any sense but anyways:
        
        Args:
            param_name (optional): the id of the parameter(s).

        Output:
            selection_list: a list of parameters.
        '''

        selection_list = []

        if param_name == '':
            return self.params
        else:
            for child in self.params:
                if param_name in child.param.values():
                    selection_list.append(child)

        return selection_list


    def writeXML(self, filename):
        '''
        Writes into the XML file.
        
        Args:
            filename: the name of the new file. If the file already exists, it will be overwritten.
        
        Output:
            0 when finished; 1 if an exception was caught.
        '''
        
        try:
            self.tree.write(filename)
        except:
            raise Exception('Could not write {}.'.format(filename))
            return 1

        return 0


    def set_instrument(self, instr, change):
        '''
        Changes the value of a given instrument in the observation attributes.
        
        Args:
            instr: the name of the instrument.
            change: the change in the attribute.
        '''

        instr = str(instr)
        for child in tree.iterfind('CONFIG/OBSERVATION'):
            for i in child:
                if i.attrib['id'] == instr:
                    i.attrib['default'] = change
                    self.full_obs[instr] = change
                    setattr(self, instr, change)


class Instrument(Observation):
    '''
    Child of Observation. An instrument branch of the xml is needed in order
    to create an object of this class.
    Attribute full_instrument: the name of the instrument present in the XML file.
    Attribute selected_instrument: a XML branch containing the data of the instrument.
    Attribute exposures_instrument: a list with all the exposures of an instrument.
    Attribute parsed_exposures_instrument: a list with all the exposure branches
    for the given instrument.
    '''

    def __init__(self, selected_instrument):

        exposures_instrument = []
        parsed_exposures_instrument = []

        for child in selected_instrument:
            exposures_instrument.append((child.tag, child.attrib))
            parsed_exposures_instrument.append(child)

        self.selected_instrument = selected_instrument
        self.full_instrument = selected_instrument.tag, selected_instrument.attrib
        self.exposures_instrument = exposures_instrument
        self.instr_name = selected_instrument.attrib['value']
        self.parsed_exposures_instrument = parsed_exposures_instrument


    def __repr__(self):
        '''
        Modifies the reference when displayed as items instead of references to
        the class.
        '''

        return str(self.full_instrument)


    def get_exposures(self, selection = ''):
        '''
        Returns either a full list of the exposures available for the
        instrument or a single Exposure class object if a selection was given.

        If you want ALL the exposures just use self.exposures.
        
        Args:
            (optional): expid of the exposure.

        Output:
            selection_list: a list with all the Exposures that match the criteria.
        '''

        selection_list = []

        if selection == '':
            for child in self.parsed_exposures_instrument:
                selection_list.append(Exposure(self.selected_instrument, child))
        else:
            for child in self.parsed_exposures_instrument:
                # for i in selection:
                #     if 'mode' in i:
                #         attribute_mode = i.split(':')[1]
                #     if 'duration' in i:
                #         attribute_duration = i.split(':')[1]
                #     if 'process' in i:
                #         attribute_process = i.split(':')[1]
                #     if 'expid' in i:
                #         attribute_expid = i.split(':')[1]
                if selection in child.attrib['expid']:
                    selection_list.append(Exposure(self.selected_instrument, child))

        return selection_list


class Exposure(Instrument):
    '''
    Information regarding the exposure for a given instrument in an
    observation.
    The constructor needs an instrument, from the list given in instrument_data.

    Attribute mode: the mode of the exposures.
    Attribute expid: the unique id of the exposure.
    Attribute duration: the duration of the exposure.
    Attribute process_exp: whether or not to process it.
    Attribute parsed_exposures: the complete list of exposures for that instrument.
    Attribute selected_exposure: the XML branch of the exposure.
    Attribute full_exposure: the full list of information for the given exposure.
    '''

    def __init__(self, selected_instrument, selected_exposure):

        products_exposure = []
        parsed_products_exposure = []
        super().__init__(selected_instrument)

        for child in selected_exposure:
            products_exposure.append((child.tag, child.attrib))
            parsed_products_exposure.append(child)

        self.selected_exposure = selected_exposure
        self.full_exposure = (selected_exposure.tag, selected_exposure.attrib)
        self.mode = selected_exposure.attrib['mode']
        self.expid = selected_exposure.attrib['expid']
        self.duration = selected_exposure.attrib['duration']
        self.process_exp = selected_exposure.attrib['process']

        # lost with the new method
        # self.exposure_instrument = instrument.attrib['value']
        self.products_exposure = products_exposure
        self.parsed_products_exposure = parsed_products_exposure


    def set_exposure(self, change):
        '''
        Updates whether or not the exposure it's processed. This file does
        NOT rewrite a xml or the existing xml file.
        
        Args: 
            change: the change to the process attribute.
        '''

        for insts in tree.iterfind('CONFIG/INSTRUMENT'):
            if self.full_instrument == (insts.tag, insts.attrib):
                for exps in insts:
                    if self.full_exposure == (exps.tag, exps.attrib):
                        self.full_exposure[1]['process'] = change
                        self.process_exp = change
                        produ.attrib['process'] = change


    def __repr__(self):
        '''
        Modifies the reference when displayed as items.
        '''

        return str(self.full_exposure)


    def get_products(self, selection = ''):
        '''
        Returns either a full list of the products available for the
        exposure or a single Exposure class object if a selection was given.

        If you want ALL the exposures just use self.exposures.
        
        Args:
            selection (optional): attributes of the exposure. Always as a tuple with:
            'name:option'. Like ('mode:fast', 'duration:32323'). No. Just write the expid.

        Output: 
            either all the exposures for the instrument (list) or a list of
        the exposures that satisfy the condition. If the list only contains one
        element, that element will be returned directly.
        '''

        selection_list = []

        if selection == '':
            for i in self.parsed_products_exposure:
                selection_list.append(Product(self.selected_instrument, self.selected_exposure, i))
        else:
            for child in self.parsed_products_exposure:
                if selection in child.attrib['value']:
                    selection_list.append(Product(self.selected_instrument, self.selected_exposure, child))

        return selection_list


class Product(Exposure):
    '''
    The products that can be obtained for a given exposure.
    The constructor needs both an instrument (the one used before) and an
    exposure from the list parsed_exposures.
    Attribute product_exposure: returns the info regarding the exposure that
    originated the products.
    Attribute full_product: all the info regarding the given product.
    Attribute selected_product: the XML branch of the given product.
    Attribute tasks_product: all the tasks inside the product.
    Attribute parsed_tasks_product: the full list with the XML branches of the
    tasks for this product.
    '''

    def __init__(self, selected_instrument, selected_exposure, selected_product):
        '''
        Basic __init___ function for the class.
        '''

        super().__init__(selected_instrument, selected_exposure)
        tasks_product = []
        parsed_tasks_product = []

        for child in selected_product:
            tasks_product.append((child.tag, child.attrib))
            parsed_tasks_product.append(child)

        self.selected_product = selected_product
        self.full_product = selected_product.tag, selected_product.attrib
        self.product_name = selected_product.attrib['value']
        self.product_process = selected_product.attrib['process']
        self.tasks_product = tasks_product
        self.parsed_tasks_product = parsed_tasks_product


    def __repr__(self):
        '''
        Modifies the reference when displayed as items.
        '''

        return str(self.full_product)


    def set_product(self, change):
        '''Change the value of a product. This file does NOT rewrite
        a xml or the existing xml file.

        Args:
            change: the desired change.
        '''

        for insts in tree.iterfind('CONFIG/INSTRUMENT'):
            if self.full_instrument == (insts.tag, insts.attrib):
                for exps in insts:
                    if self.full_exposure == (exps.tag, exps.attrib):
                        for produ in exps:
                            if self.full_product == (produ.tag, produ.attrib):
                                self.full_product[1]['process'] = change
                                produ.attrib['process'] = change


    def get_tasks(self, name = '', purpose = ''):
        '''
        Selects certain tasks. If both name and purpose are left without
        arguments, all the tasks will be returned in a dictionary format.
        
        Args:
            name (optional): the name of the task.
            purpose (optional): the purpose of the task.
        
        Output: 
            either:
        ----- a list containing XML branches of the matching products.
        ----- an object of Task class if there's only one match.
        ----- if no parameters are given, the dictionary response of all the tasks.
        '''

        selection_list = []

        for child in self.parsed_tasks_product:
            if name != '' and purpose == '':
                if name in child.attrib['name']:
                    selection_list.append(Task(self.selected_instrument, self.selected_exposure, self.selected_product, child))
            if name == '' and purpose != '':
                if purpose in child.attrib['purpose']:
                    selection_list.append(Task(self.selected_instrument, self.selected_exposure, self.selected_product, child))
            if name != '' and purpose != '':
                if name in child.attrib['name'] and purpose in child.attrib['purpose']:
                    selection_list.append(Task(self.selected_instrument, self.selected_exposure, self.selected_product, child))
            if name == '' and purpose == '':
                selection_list.append(Task(self.selected_instrument, self.selected_exposure, self.selected_product, child))


        return selection_list


    def new_product_name(self):
        '''
        Returns the appropiate filename for a given product according to its 
        instrument and exposure.

        Ougput:
            the name of the file as a string.
        '''

        new_product_filename = self.instr_name + '_' + self.expid + '_' + self.product_name + '.fits'

        return new_product_filename


class Task(Product):
    '''
    Purpose and name of the tasks that can be performed with a given product.
    The constructor needs the instrument, exposure and product, and they must be
    coherent between them.

    Attribute purpose: describes the purpose of a given task.
    Attribute name: simply the name of the task.
    Attribute parsed_tasks: list with the total amount of tasks for a given product.
    '''

    def __init__(self, selected_instrument, selected_exposure, selected_product, selected_task):
        super().__init__(selected_instrument, selected_exposure, selected_product)
        params_task = []
        parsed_params_task = []

        for child in selected_task:
            params_task.append((child.tag, child.attrib))
            parsed_params_task.append(child)

        self.selected_task = selected_task
        self.full_task = selected_task.tag, selected_task.attrib
        self.params_task = params_task
        self.task_name = selected_task.attrib['name']
        self.task_purpose = selected_task.attrib['purpose']
        self.parsed_params_task = parsed_params_task


    def __repr__(self):
        '''
        Modifies the reference when displayed as items.
        '''

        return str(self.full_task)


    def get_params(self, param_name = ''):
        '''
        Returns the value of the parameter from the XML according to the
        selection (if any).

        Args:
             param_name (optional): the name of the parameter.
        
        Output:
            a object of the Param class.
        '''

        selection_list = []
        if param_name == '':
            for child in self.parsed_params_task:
                selection_list.append(Param(self.selected_instrument, self.selected_exposure, self.selected_product, self.selected_task, child))
        else:
            for child in self.parsed_params_task:
                if param_name in child.attrib['id']:
                    selection_list.append(Param(self.selected_instrument, self.selected_exposure, self.selected_product, self.selected_task, child))

        return selection_list


    def run_task(self, param_list):
        '''
        Runs a task using the parameters passed in the argument.

        Args:
            param_list: the list of the parameters. Expected Param objects.
        '''

        inargs = []
        for param in param_list:
            inargs.append(param.param_name + '=' + param.param_value)
        
        t = W(self.task_name, inargs)
        print(self.task_name, inargs)
        t.run()


class Param(Task):
    '''
    Class defines the parameters for a given task.
    The constructore requires an instrument, an exposure, a product an a task.
    All of them must be coherent.

    Attribute selected_param: the XML branch with the parameter.
    Attribute full_param: all the info regarding the parameter object.
    '''

    def __init__(self, selected_instrument, selected_exposure, selected_product, selected_task, selected_param):
        super().__init__(selected_instrument, selected_exposure, selected_product, selected_task)
        self.selected_param = selected_param
        self.full_param = selected_param.tag, selected_param.attrib
        self.param = selected_param.attrib
        
        try:
            self.param_name = selected_param.attrib['id']
            self.param_value = selected_param.attrib['default']
        except KeyError:
            self.param_name = ''
            self.param_value = ''


    def __repr__(self):
        '''
        Modifies the reference when displayed as items.
        '''

        return str(self.full_param)


    def set_param(self, change):
        '''
        Change the value of the object parameter. This file does NOT rewrite
        a xml or the existing xml file.
        
        Args:
            change: the change to be substitued.
        '''

        for insts in tree.iterfind('CONFIG/INSTRUMENT'):
            if self.full_instrument == (insts.tag, insts.attrib):
                for exps in insts:
                    if self.full_exposure == (exps.tag, exps.attrib):
                        for produ in exps:
                            if self.full_product == (produ.tag, produ.attrib):
                                for task in produ:
                                    if self.full_task == (task.tag, task.attrib):
                                        for parameter in task:
                                            if self.full_param == (parameter.tag, parameter.attrib):
                                                self.full_param[1]['default'] = change
                                                parameter.attrib['default'] = change
